/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.builder;

import java.util.*;
import java.io.*;
import org.aspectj.ajde.*;
import org.eclipse.ajdt.internal.core.AJDTEventTrace;
import org.eclipse.ajdt.internal.ui.*;
import org.eclipse.ajdt.internal.ui.ajde.*;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.IVMRunner;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.VMRunnerConfiguration;
//import org.eclipse.jdt.launching.VMRunnerResult;

/**
 * This Builder selects the appropriate .lst (build config file) for
 * the project and then invokes the ajc compiler.
 * No incremental compilation support as yet - recommend turning
 * the "perform build automatically on resource modification" option
 * off under workbench->preferences.
 */
public class Builder extends IncrementalProjectBuilder {

	/**
	 * The name of the default build config file for an AspectJ project
	 */
	public static final String DEFAULT_CONFIG_FILE = "default.lst";

	/**
	 * Code returned when a failure occurs creating a build config file
	 */
	private static final int CONFIG_FILE_WRITE_ERROR = -10;

	/**
	 * Seconds to wait for a build to finish
	 */
	private static final int COMPILE_WAIT_TIME = 60;

	/**
	 * The last project we did a build for, needed by content outline
	 * view to decide which updates to accept.
	 */
	private static IProject lastBuiltProject = null;

	/**
	 * Constructor for AspectJBuilder
	 */
	public Builder() {}

	/**
	 * What did we last build?
	 */
	public static IProject getLastBuildTarget( ) {
		return lastBuiltProject;
	}

	/**
	 * @see IncrementalProjectBuilder#build(int, Map, IProgressMonitor)
	 * kind is one of: FULL_BUILD, INCREMENTAL_BUILD or AUTO_BUILD
	 * currently we do a full build in every case!
	 */
	protected IProject[] build(
		int kind,
		Map args,
		IProgressMonitor progressMonitor)
		throws CoreException {
		try {
			if ( kind == AUTO_BUILD && AspectJPreferences.isAutobuildSuppressed() ) {
				return null;
			}
			
			AspectJPlugin ajPlugin = AspectJPlugin.getDefault();

			IProject project = getProject();
			ajPlugin.setCurrentProject(project);
			AJDTEventTrace.build( project, ajPlugin.getBuildConfigurationFile( project ));

			ProjectProperties props = ajPlugin.getAjdtProjectProperties();
			List projectFiles = props.getProjectSourceFiles( project );
			
			// AJDE bug workaround
			if ( projectFiles.isEmpty() ) { return null; }
			
			props.clearMarkers();

			CompilerMonitor compilerMonitor = ajPlugin.getCompilerMonitor();
			compilerMonitor.prepare(projectFiles, progressMonitor);

			updateBuildConfigIfNecessary(project, projectFiles);
			lastBuiltProject = project;
			try {
				Ajde.getDefault().getBuildManager().build(getBuildFilePath(project));
			} catch (Throwable t) {
				t.printStackTrace();
			}

			waitForBuildCompletion(compilerMonitor);

		} catch (Throwable t) {
			Ajde.getDefault().getErrorHandler().handleError("Compile failed.", t);
		}

		IProject[] requiredResourceDeltasOnNextInvocation = null;
		return requiredResourceDeltasOnNextInvocation;
	}

	/**
	 * Wait up to COMPILE_WAIT_TIME seconds for a build to finish
	 */
	private void waitForBuildCompletion(CompilerMonitor monitor) {
		int timesTried = 0;
		while (!monitor.finished() && timesTried < COMPILE_WAIT_TIME) {
			timesTried++;
			try {
				Thread.sleep(1000);
			} catch (Exception e) {}

		}
		if (timesTried == COMPILE_WAIT_TIME) {
			//System.err.println("Failed to receive finish() call after 20 seconds...");
			monitor.finish();
		}
	}

	/**
	 * Get the fully qualified path to the build configuation file specified
	 * for this project.
	 */
	private String getBuildFilePath(IProject project) {
		String buildfile = AspectJPlugin.getBuildConfigurationFile(project);
		return buildfile;
	}

	/**
	 * If the project uses a default build file, then it will be regenerated
	 * to ensure that all project files are included.
	 */
	private void updateBuildConfigIfNecessary(IProject project, List projectFiles)
		throws CoreException {
		if (getBuildFilePath(project).endsWith(DEFAULT_CONFIG_FILE)) {
			writeBuildConfigFile(projectFiles, project);
		}
	}

	/**
	 * Create a full build configuration file for this project
	 */
	private void writeBuildConfigFile(List projectFiles, IProject project)
		throws CoreException {
		String configurationFilename = getBuildFilePath(project);

		try {
			FileWriter fw = new FileWriter(configurationFilename);
			BufferedWriter bw = new BufferedWriter(fw);
			for (Iterator it = projectFiles.iterator(); it.hasNext();) {
				File jf = (File) it.next();
				String fileName = jf.toString();
				if (fileName.endsWith(".java") || fileName.endsWith(".aj")) {
					bw.write(fileName);
					bw.write("\n");
				}
			}
			bw.flush();
			fw.flush();
			fw.close();
		} catch (Exception e) {
			Status status =
				new Status(
					Status.ERROR,
					AspectJPlugin.PLUGIN_ID,
					CONFIG_FILE_WRITE_ERROR,
					AspectJPlugin.getResourceString("configFileCreateError"),
					e);
			throw new CoreException(status);
		}

	}
}