/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.core.resources;

import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;

import org.aspectj.ajde.Ajde;
import org.aspectj.ajde.ui.AbstractIconRegistry;
import org.aspectj.ajde.ui.AbstractIcon;
import org.aspectj.ajde.ui.BuildConfigNode;
import org.aspectj.asm.ProgramElementNode;
import org.aspectj.asm.Relation;
import org.aspectj.asm.ProgramElementNode.Kind;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
/**
 * Utility class providing access to all the images used by this
 * plugin.
 */
public class AspectJImages extends AbstractIconRegistry {

	private static AspectJImages instance;

	private static final org.eclipse.ui.ISharedImages workbenchImages = 
		AspectJPlugin.getDefault().getWorkbench().getSharedImages();
		
	private static final URL ajdeIconLocation = 
		Platform.getPluginRegistry().getPluginDescriptor( "org.aspectj.ajde" ).getInstallURL();

	private static final String AJDE_ICON_PATH_PREFIX = "org/aspectj/ajde/resources/";
	private static final String AJDE_IMPORT_RELATION = "imports";

	// The following icons are private and should be accessed through the 
	// getStructureIcon operation:
	private final AbstractIcon JDT_PACKAGE = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_PACKAGE ) );
	private final AbstractIcon JDT_FILE = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_CUNIT ) );
	private final AbstractIcon JDT_CLASS = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_CLASS ) );
	private final AbstractIcon JDT_INTERFACE = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_INTERFACE ) );
	private final AbstractIcon JDT_PRIVATE = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_PRIVATE ) );
	private final AbstractIcon JDT_PROTECTED = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_PROTECTED ) );
	private final AbstractIcon JDT_PUBLIC = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_PUBLIC ) );
	private final AbstractIcon JDT_DEFAULT = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_DEFAULT ) );
	private final AbstractIcon JDT_IMPORTS = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_IMPCONT ) );
	private final AbstractIcon WKBENCH_FILE = new AJDTIcon(
		workbenchImages.getImageDescriptor( workbenchImages.IMG_OBJ_FILE ) );
	private final AbstractIcon WKBENCH_FOLDER = new AJDTIcon(
		workbenchImages.getImageDescriptor( workbenchImages.IMG_OBJ_FOLDER ) );
	private final AbstractIcon AJDT_ASPECT = new AJDTIcon(
		"icons/structure/aspect.gif" );

	// These icons are publically available
	public static final AJDTIcon WKBENCH_INFO = new AJDTIcon(
		workbenchImages.getImageDescriptor( workbenchImages.IMG_OBJS_INFO_TSK) );
	public static final AJDTIcon JDT_IMPORTED = new AJDTIcon(
		JavaUI.getSharedImages().getImageDescriptor( ISharedImages.IMG_OBJS_IMPDECL ) );
		
	public static final AJDTIcon E_ADVICE           = new AJDTIcon(
				"icons/structure/e_advice.gif");
	public static final AJDTIcon E_DECLARE_PARENTS  = new AJDTIcon(
				"icons/structure/e_declareParents.gif");
	public static final AJDTIcon E_INTRODUCTION     = new AJDTIcon(
				"icons/structure/e_introduction.gif");
	public static final AJDTIcon E_JOINPOINT        = new AJDTIcon(
				"icons/structure/e_joinPoint.gif");
	public static final AJDTIcon G_ADVICE           = new AJDTIcon(
				"icons/structure/g_advice.gif");
	public static final AJDTIcon G_DECLARE_PARENTS  = new AJDTIcon(
				"icons/structure/g_declareParents.gif");
	public static final AJDTIcon G_INTRODUCTION     = new AJDTIcon(
				"icons/structure/g_introduction.gif");
	public static final AJDTIcon G_JOINPOINT        = new AJDTIcon(
				"icons/structure/g_joinPoint.gif");


	public static AspectJImages registry( ) {
		if ( instance == null ) {
			instance = new AspectJImages( );
		}
		return instance;	
	}

	// Methods inherited from AbstractIconRegistry

	/**
	 * @see AbstractIconRegistry#createIcon(String)
	 */
	protected AbstractIcon createIcon(String path) {
		if ( path.startsWith( AJDE_ICON_PATH_PREFIX ) ) {
			path = path.substring( AJDE_ICON_PATH_PREFIX.length( ) );
		}
		AJDTIcon retVal = null;
		try {
			URL url = new URL( ajdeIconLocation, "icons/" + path );
			retVal = new AJDTIcon( url );
		} catch ( MalformedURLException malEx ) {
			retVal = AJDTIcon.MISSING_ICON;
			AspectJPlugin.getDefault().getErrorHandler().handleError( 
				"Missing image resource: icons/" + path, malEx );
			}
		return retVal;
	}

	/**
	 * We override this method to provide the eclipse-specific icons
	 * for certain "Kinds" of resource.
	 * @see AbstractIconRegistry#getStructureIcon(Kind)
	 */
	public AbstractIcon getStructureIcon(Kind kind ) {
		if ( kind == ProgramElementNode.Kind.CONSTRUCTOR ||
			 kind == ProgramElementNode.Kind.METHOD ||
			 kind == ProgramElementNode.Kind.FIELD ) {
			 	String err = "Should use 2-arg version of getStructureIcon for contructors, methods and fields.";
			 	System.err.println( err ); 
				throw new RuntimeException( err );			 		
			 }
		if (kind == ProgramElementNode.Kind.PACKAGE) {
			return JDT_PACKAGE;
		} else if (kind == ProgramElementNode.Kind.FILE) {
			return WKBENCH_FILE;
		} else if (kind == ProgramElementNode.Kind.FILE_JAVA) {
			return JDT_FILE;
		} else if (kind == ProgramElementNode.Kind.CLASS) {
			return JDT_CLASS;
		} else if (kind == ProgramElementNode.Kind.INTERFACE) {
			return JDT_INTERFACE;
		} else if (kind == ProgramElementNode.Kind.ASPECT) {
			return AJDT_ASPECT;
		}

		AbstractIcon icon = super.getStructureIcon( kind );
		if ( icon != null ) { return icon; }
		else { return AJDTIcon.MISSING_ICON; }
	}


	/**
	 * This method added by AMC for use by BuildConfigEditor
	 */
	public AbstractIcon getStructureIcon( BuildConfigNode.Kind kind ) {
		if ( kind == BuildConfigNode.Kind.FILE_ASPECTJ ) {
			return AJDT_ASPECT;
		} else if ( kind == BuildConfigNode.Kind.FILE_JAVA ) {
			return JDT_FILE;
	    } else if ( kind == BuildConfigNode.Kind.DIRECTORY ) {
			return WKBENCH_FOLDER;
		} else if ( kind == BuildConfigNode.Kind.FILE_LST ) {
			return FILE_LST;
		} else if ( kind == BuildConfigNode.Kind.ERROR ) {
			return WKBENCH_FILE;
		} else {
			return AJDTIcon.MISSING_ICON;
		}				
	}

	/**
	 * Added this method for constructor, method, field to differentiate
	 * icon based on Accessbility
	 */
	public AbstractIcon getStructureIcon( Kind kind, ProgramElementNode.Accessibility access ) {
		// eclipse uses the same icons regardless of kind...
		if ( kind == ProgramElementNode.Kind.CONSTRUCTOR ||
			 kind == ProgramElementNode.Kind.METHOD ||
			 kind == ProgramElementNode.Kind.FIELD ) {
			if ( access == ProgramElementNode.Accessibility.PUBLIC ) {
				return JDT_PUBLIC;
			} else if ( access == ProgramElementNode.Accessibility.PROTECTED ) {
				return JDT_PROTECTED;
			} else if ( access == ProgramElementNode.Accessibility.PRIVATE ) {
				return JDT_PRIVATE;
			} else if ( access == ProgramElementNode.Accessibility.PACKAGE ) {
				return JDT_DEFAULT;
			} else if ( access == ProgramElementNode.Accessibility.PRIVILEGED ) {
				return JDT_DEFAULT; //?? what to do here ??
			} else return AJDTIcon.MISSING_ICON;
		} else  {
			return getStructureIcon( kind );
		}
	}	

	/**
	 * Needs to provide imports icons in addition to those provided by
	 * superclass.
	 * @see AbstractIconRegistry#getRelationIcon(Relation)
	 */
	public AbstractIcon getRelationIcon(Relation rel) {
		AbstractIcon retVal = null;
		if ( rel.getForwardNavigationName().equals( AJDE_IMPORT_RELATION ) ) {
			retVal = JDT_IMPORTS;
		} else {
			retVal = super.getRelationIcon(rel);
		}
		if ( retVal == null ) { retVal = AJDTIcon.MISSING_ICON; }
		return retVal;
	}

}
