/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.editor;


import org.eclipse.ajdt.ui.AspectJPlugin; 
import org.eclipse.jdt.internal.ui.javaeditor.*;
import org.eclipse.jdt.internal.ui.*;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.texteditor.*;
import org.eclipse.jdt.ui.text.JavaSourceViewerConfiguration;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.*;

import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;

import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.jdt.ui.*;
import org.eclipse.jdt.core.*;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;
import org.eclipse.ui.texteditor.MarkerAnnotation;
import org.eclipse.ui.views.contentoutline.*;
import org.eclipse.ui.texteditor.MarkerUtilities;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.eclipse.ui.views.tasklist.TaskList;

import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.IFileEditorInput;

import org.eclipse.ajdt.internal.core.AJDTEventTrace;
import org.eclipse.ajdt.internal.ui.AspectJPreferences;

/**
 * AspectJ Editor extends internal JDT editor in order to use our TextTools.
 * Better would be a clean interface in JDT to achieve the same effect.
 * We also override getAdapter( ) to plug-in the AspectJ-aware outline view.
 */
public class AspectJEditor extends CompilationUnitEditor {

	private AspectJContentOutlinePage contentOutlinePage;

	/**
	 * Constructor for AspectJEditor
	 */
	public AspectJEditor() {
		super();
				
		AspectJTextTools textTools= AspectJPlugin.getDefault().getAspectJTextTools();
		setSourceViewerConfiguration(new JavaSourceViewerConfiguration(textTools, this));
		
	}

	/**
	 * Get an adapter - if they want an outliner, give them ours!
	 * Other adapters we could consider providing:
	 * <ol>
	 * <li>ITextOperationTarget</li>
	 * <li>IFindReplaceTarget</li>
	 * </ol>
	 */
	public Object getAdapter( Class key ) {
		//System.err.println( "Asked for adapter: " + key.getName() );
		if ( key.equals( IContentOutlinePage.class ) ) {
			return getContentOutlinePage( key );			
		}		
		return super.getAdapter( key );
	}


	/**
	 *  Override of doSave to comment-out call to getStatusLineManager - 
	 * always returns null (why?) in our environment.
	 * Also ask the contentOutlinePage to update - not strictly required at the
	 * moment, but will be in the future.
	 */
	public void doSave(IProgressMonitor progressMonitor) {
		
		IDocumentProvider p= getDocumentProvider();
		if (p == null)
			return;
			
		if (p.isDeleted(getEditorInput())) {
			
			if (isSaveAsAllowed()) {
				
				/*
				 * 1GEUSSR: ITPUI:ALL - User should never loose changes made in the editors.
				 * Changed Behavior to make sure that if called inside a regular save (because
				 * of deletion of input element) there is a way to report back to the caller.
				 */
				 performSaveAs(progressMonitor);
			
			} else {
				
				/* 
				 * 1GF5YOX: ITPJUI:ALL - Save of delete file claims it's still there
				 * Missing resources.
				 */
				Shell shell= getSite().getShell();
				MessageDialog.openError(shell, "CompilationUnitEditor.error.saving.title1", "CompilationUnitEditor.error.saving.message1"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
		} else {	
			
			//getStatusLineManager().setErrorMessage(""); //$NON-NLS-1$
			
			IWorkingCopyManager manager= JavaPlugin.getDefault().getWorkingCopyManager();
			ICompilationUnit unit= manager.getWorkingCopy(getEditorInput());
			
			if (unit != null) {
				synchronized (unit) { 
					performSaveOperation(createSaveOperation(false), progressMonitor); 
				}
			} else 
				performSaveOperation(createSaveOperation(false), progressMonitor);
		}
		if (contentOutlinePage == null) {
			Object outlinePage = getContentOutlinePage( IContentOutlinePage.class );		
			if (outlinePage instanceof AspectJContentOutlinePage )
			  contentOutlinePage = (AspectJContentOutlinePage)outlinePage;	
		}
		if (contentOutlinePage != null) 
		 contentOutlinePage.update( );
	}


	/**
	 * Get the content outline page - either the JDT one or our own
	 * depending on the preference setting.
	 */
	private Object getContentOutlinePage( Class key ) {
		Object outlinePage = null;

		if ( AspectJPreferences.isAspectJOutlineEnabled() ) {
			IEditorInput input = getEditorInput( );
			if ( input instanceof IFileEditorInput ) {
				IFile f = ((IFileEditorInput)input).getFile( );
				IProject p = f.getProject();
				try {
					if ( p.hasNature( AspectJPlugin.ID_NATURE ) ) {
						contentOutlinePage = new AspectJContentOutlinePage(
						  this,
						  f
						);	
						outlinePage = contentOutlinePage;
					} else { outlinePage = super.getAdapter( key ); }
				} catch ( CoreException cEx ) {
					outlinePage = super.getAdapter( key );
				}
			} else {
				outlinePage = super.getAdapter( key );		
			}
		} else {
			outlinePage = super.getAdapter( key );
		}

		return outlinePage;
	}


	public void doSetInput( IEditorInput input) throws CoreException {
		super.doSetInput( input );
		if ( input instanceof IFileEditorInput ) {
			IFileEditorInput fInput = (IFileEditorInput) input;
			AJDTEventTrace.editorOpened( fInput.getFile() );
		}
	}
	
	public void dispose( ) {
		IEditorInput input = getEditorInput();
		if ( input instanceof IFileEditorInput ) {
			IFileEditorInput fInput = (IFileEditorInput) input;
			AJDTEventTrace.editorClosed( fInput.getFile() );
		}
		super.dispose();				
	}	
	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		// AMC -added this method to catch user switching between multiple open files
		// in the editor - used to trigger a structured selection event, but now
		// triggers a TextSelection event. TextSelection event does not give AJP
		// enouhg info to determine project, so have to do from here instead.
		//System.out.println( "Focus given to: " + getEditorInput( ).getName() );
		IEditorInput input = getEditorInput();
		if ( input instanceof IFileEditorInput ) {
			IFileEditorInput fInput = (IFileEditorInput) input;
			AspectJPlugin.getDefault().setCurrentProject( fInput.getFile().getProject() );
		}
		super.setFocus();
	}

}


